#include "inet.h"
#include "segment.h"

/* functions defined in this module */
/*int CommInit(void);
void CommQuit(); */
int GetDataSize(int, double, double, double, double, int* ,
		int* , int* , int*, void** );
int RemoteData(int, int, int, int, int, int, void* , void** );

/* external functions */
extern int      errno;
extern char    *sys_errlist[];
extern int      ReadShort2Float(int, float *, int);
extern int      ReadInt(int, int *, int);
extern int      ReadDouble (int, double *, int);
extern int      ReadBytes(int, char *, int);
extern int      WriteBytes(int, char *, int);
extern int      WriteShort(int, short *, int);
extern int      WriteInt(int, int *, int);
extern int      WriteDouble(int, double *, int);

/* globals for socket numbers */
static int datasock, ctrlsock; /* data and control socket descriptors */

/* ----------------------------------------------------------------------
Name: CommInit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------ */
int CommInit()  {

   char            hostname[MAXHOSTNAMELEN];

   strcpy(hostname, SERV_HOSTNAME);
   if ((datasock = ClientInit(hostname, "data")) == ERROR)
      return (CONNECT_ERROR);
   if ((ctrlsock = ClientInit(hostname, "control")) == ERROR)
   {
      close(datasock);
      return (CONNECT_ERROR);
   }

   return (OK);
}



/* ----------------------------------------------------------------------
Name: CommInit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int GetDataSize(int data_type, double south, double west,
		double north, double east, int *start_row,
		int *start_column, int *rows, int *columns, void **refptr)
{

   int             service_type[2];
   char            clearance[BUFLEN];
   double          coord[4];
   int             row_column[4];

/* request type of service from the central rendering server */
   service_type[0] = 1;      /* get data size */
   service_type[1] = data_type;    /* type of data */
   if (south == -1)
     service_type[0] = -1;   /* send done signal to server to clean up */
   printf("Requesting service type %d from the server\n", data_type);
   if (WriteInt(ctrlsock, service_type, 2) != 2)
   {
      printf("ERROR: In writing service request to control channel\n");
      return (ERROR);
   }

/* wait for server to initialize */
   printf("Waiting for server clearance\n");
   if (ReadBytes(ctrlsock, clearance, BUFLEN) != BUFLEN)
   {
      printf("ERROR: In reading server signal from control channel\n");
      return (ERROR);
   }

/* pack coordinates into coord buffer and beam it thru the data socket */
   coord[0] = south;
   coord[1] = west;
   coord[2] = north;
   coord[3] = east;

   printf("Transmitting map coordinates\n");
   if (WriteDouble(datasock, coord, 4) != 4)
   {
      printf("ERROR: In writing coordinates to data channel\n");
      return (ERROR);
   }

/* read map segment row column data from server */
   if (ReadInt(datasock, row_column, 4) != 4)
   {
      printf("ERROR: In reading row column data from control channel\n");
      return (ERROR);
   }
   *start_row = row_column[0];
   *start_column = row_column[1];
   *rows = row_column[2];
   *columns = row_column[3];

/* debug */
	printf("start_row = %d start_column = %d rows = %d columns = %d\n", 
	        *start_row, *start_column, *rows, *columns);

/* error check */
   if (*start_row == -1 * DATA_ERROR)
      return (DATA_ERROR);       /* connected to server but could not extract
                                   data */
   else if (*start_row == -1 * RANGE_ERROR)
      return (RANGE_ERROR);      /* requested data is out of range */

   return (OK);

}




/* ----------------------------------------------------------------------
Name: CommInit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int RemoteData(int data_type, int start_row, int start_column,
	       int rows, int columns, int scale, void *data, 
	       void **refptr)  {

   int             service_type[2];
   char            clearance[BUFLEN];
   int             row_column[5];  /* scale factor included here */
   int             status, nbytes, nread, size, nelements;

/* request type of service from the central rendering server */
   service_type[0] = 2;    /* get data */
   service_type[1] = data_type;  /* of this type */
   printf("Requesting service type %d from the server\n", data_type);
   if (WriteInt(ctrlsock, service_type, 2) != 2)
   {
      printf("ERROR: In writing service request to control channel\n");
      return (ERROR);
   }

/* wait for server to initialize */
   printf("Waiting for server clearance\n");
   if (ReadBytes(ctrlsock, clearance, BUFLEN) != BUFLEN)
   {
      printf("ERROR: In reading server signal from control channel\n");
      return (ERROR);
   }

/* pack row column data into row column buffer and beam it thru
   the data socket */
   row_column[0] = start_row;
   row_column[1] = start_column;
   row_column[2] = rows;
   row_column[3] = columns;
   row_column[4] = scale;
   printf("Transmitting row column positions\n");
   if (WriteInt(datasock, row_column, 5) != 5)
   {
      printf("ERROR: In writing row column positions to data channel\n");
      return (ERROR);
   }

/* read request service status */
   if (ReadInt(ctrlsock, &status, 1) != 1)
     {
       printf("ERROR: In reading server request service flag from control
            channel\n");
       return (ERROR);
     }
   if ((status == DATA_ERROR) || (status == RANGE_ERROR))
     return (status);

/* read map segment data from server */
   if (data_type == DEM)
     {                            /* elevation data */
       nelements = rows * columns;
       if ((nread = ReadShort2Float(datasock, 
				    (float *) data, nelements)) != nelements)
	 {
	   printf("ERROR: In reading map segment data from data channel\n");
	   return (ERROR);
	 }
       printf("Read %d elevation data from socket\n", nread);
     }
   else
     {                            /* image data */
       size = ((Misc *)(*refptr))->datasize;
       if ((nbytes = ReadBytes(datasock, (char *) data, size)) != size)
	 {
	   printf("ERROR: In reading map segment data from data channel\n");
	   return (ERROR);
	 }
      printf("Read %d bytes from socket\n", nbytes);
     }

   return (OK);

}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void CommQuit()
{

/* close all socket streams */
   close(datasock);
   close(ctrlsock);

   printf("Goodbye\n");

}
